"""
.. module:: __init__.py

__init__.py
*************

:Description: __init__.py

    

:Authors: bejar
    

:Version: 

:Created on: 16/06/2016 9:23 

"""

from .BorderDetection import QiuCaoBorder

__author__ = 'bejar'

__all__ = ['QiuCaoBorder']